
# This file was generated by 'versioneer.py' (0.18) from
# revision-control system data, or from the parent directory name of an
# unpacked source archive. Distribution tarballs contain a pre-generated copy
# of this file.

import json

version_json = '''
{
 "date": "2020-10-23T08:52:25-0400",
 "dirty": false,
 "error": null,
 "full-revisionid": "40c8a35486a92a9d9f1ee3fd74a6606c8a82fd2f",
 "version": "4.12.0"
}
'''  # END VERSION_JSON


def get_versions():
    return json.loads(version_json)
